/*** FrontPanel_JS.js - referenced in FrontPanel.html ***/
var DEBUG_ON = 0;

/************************************************************/
/*** NEWLY ADDED VARIABLE ***/
/************************************************************/
var RUN_TASK_INTERVAL = 100;	// in ms
var IMAGE_REFRESH_RATE = 1000;	// in ms
var IMAGE_REFRESH_TIMEOUT_RATE = 20000;	// in ms
var IMAGE_REFRESH_COUNT = (IMAGE_REFRESH_RATE / RUN_TASK_INTERVAL);
var IMAGE_REFRESH_TIMEOUT_COUNT = (IMAGE_REFRESH_TIMEOUT_RATE / IMAGE_REFRESH_RATE);
var ERROR_RETRY_COUNT = 2;

var FP_EVENT_IDLE = 0;
var FP_EVENT_BUSY = 1;
var FP_EVENT_RETRY = 2;
var FP_EVENT_KEY = 3;

var IMG_NORMAL = 0;
var IMG_PRINTER_FRIENDLY = 1;

var imgType = IMG_NORMAL;
var imgFileName = "";
	
var totaltime = 0; // time between refreshes
var postTimeoutCount = 0;
var errorRetryCount = 0;
var aspErrorCount = 0;
var aspErrorString = "";
var fpEvent = FP_EVENT_IDLE;
var prevCommand = "";
var runThreadId = 0;
var runClosedFlag = 1;
var f_Thread = 0;

/********************/
/*** DEBUG WINDOW ***/
/********************/

/**** Sends a message to the debug window */
function debug(txt, mod){
	/* Only try to print if debug switch is 1 */
	if(DEBUG_ON){
		console.log(Date() + "[" + (new Date()).getMilliseconds() + "]" + " : " + txt);
	}
}


/********************************************************/
/*** FUNCTIONS USED TO MAINTAIN THE FRONT PANEL QUEUE ***/
/********************************************************/

/* The queue holds information about the virtual/keyboard keys
   the user pressed */

/* note:  the front panel queue is an element on a hidden form created in
   FrontPanel.html */

/**** Place the button on the front panel queue ****/
function pushFrontPanelQueue(button) {

	if(document.hiddenform.userinput.value == "none"){
		document.hiddenform.userinput.value = "";
	}
	
	document.hiddenform.userinput.value = document.hiddenform.userinput.value + button;
	
	return;
}

/**** Remove all button and clicks from the queue ****/
function emptyFrontPanelQueue() {
	document.hiddenform.userinput.value = "none";
}

function FailedCommunication(alertFlag) {
	myNode = document.getElementById("screen_container");
	try {
		while (myNode.firstChild) {
			myNode.removeChild(myNode.firstChild);
		}
	}
	catch (e) {
	}
	var app = document.createElement("p");
	app.setAttribute("id","screen");				
	document.getElementById('screen_container').appendChild(app);
	
	switch(alertFlag)
	{
		case 1:
		{
			console.log("Lost Connection");
			document.getElementById("screen").innerHTML = 'Cannot connect to the instrument.<br>Click the Browser Control Instrument tab to retry.';
		}
			break;
			
		case 2:
		{
			console.log(aspErrorString);
			document.getElementById("screen").innerHTML = 'Cannot connect to the instrument.';
			alert("ERROR!!!\n\nDescription:\n" + aspErrorString);
		}
			break;
	}	
}


/****************************************************************/
/*** FUNCTIONS TO HANDLE THE VIRTUAL KEY AND KEYBOARD PRESSES ***/
/****************************************************************/
var TRANSPARENT = "Images/transparent.gif";

/**** This is called when a key press event occurs.  The key is placed
      on the front panel queue ****/
function cycleClick(el){
	
	/* Get the button */
	var button = el.id;
		
	/* Put the command on the server queue */
	pushFrontPanelQueue(button);

}

/************************************************************/
/*** NEWLY ADDED FUNCTION ***/
/************************************************************/
function FrontPanel_JS_Init() {
	
    StartRun();
    SetupScanButton();
}

var pressTimer;
var longpress = false;

var scanButton = document.getElementById("Scan");
var scanStopButton = document.getElementById("ScanStop");

function SetupScanButton() {
    scanButton.addEventListener('click', function() {
    	if (!longpress) {	// if not long press, perform normal operation
    		cycleClick(scanButton);
    	}
    	return false;
    });

    scanButton.addEventListener('mouseup', function(e) {
    	clearTimeout(pressTimer);
    	return false;
    });

    scanButton.addEventListener('mousedown', function(e) {
    	longpress = false;
    	pressTimer = window.setTimeout(function() {
    		longpress = true;
    		cycleClick(scanStopButton);
    	}, 2000);
    	return false;
    });
}

/** THREAD SUPPORT
*		The run() method is called when the thread is started. If
* your application performs any ongoing activities without waiting for user
* input, the code for implementing that behavior typically goes here. For
* example, for an application that performs animation, the run() method controls
* the display of images.
*/
function run(){
	
	var command = document.hiddenform.userinput.value; // contains key clicks make on web virtual front panel

	if(command == null || command == "none")
	{
		command = "";
	}

	if (command.length != 0) 
	{
		if(RequestNewScreen(command, 0) != 1)
		{
			emptyFrontPanelQueue();
		}
		totaltime = 0;
	}
	else
	{
		totaltime++;
		if (totaltime >= IMAGE_REFRESH_COUNT) 
		{
			RequestNewScreen(command, 1);
			totaltime = 0;
		}
	}
			
}

/**
* Handles the request to the instrument for a new image
*
* @param command String
* @return boolean
*/
function RequestNewScreen(command, flag)
{
	var toWrite = "";	
	
	if(fpEvent == FP_EVENT_BUSY)
	{
		maxPostTimeoutCount = IMAGE_REFRESH_TIMEOUT_COUNT;
		if(!flag)
		{
			maxPostTimeoutCount = (IMAGE_REFRESH_TIMEOUT_RATE / RUN_TASK_INTERVAL);
		}
		
		postTimeoutCount++;
		if(postTimeoutCount >= maxPostTimeoutCount)
		{
			postTimeoutCount = maxPostTimeoutCount;
			CloseRun(1);
		}
		return 1;
	}
	else if(fpEvent == FP_EVENT_RETRY)
	{
		if(command == "")
		{
			command = prevCommand;
		}
	}
	postTimeoutCount = 0;
	
	if(command != "")
	{				
		toWrite = command;				
	}	
	prevCommand = command;
	
	SetKeyEvent(toWrite);
	
	return 0;
}


/**
* Takes the image buffer above and populates the image
*/

function SetKeyEvent(command)
{
	var dataInput = "";
	
	if(command != "")
	{
		dataInput = {queryInput: command, queryMode: "SetKey"};
	}

	GetImageEx(dataInput);
}

function updateImage(obj)
{
	if (obj.Img == "Image Not Ready") {
		StartThread();
		console.log(Date() + " : " + obj.Img);
		fpEvent = FP_EVENT_RETRY;
	}
	else if (parseInt(obj.Img) > 0) {
		document.getElementById("screen").src = "Images\\CE_ScreenShot" + obj.Img + "\.bmp?" + new Date().getTime();
		fpEvent = FP_EVENT_IDLE;
	}
	else {
		console.log(Date() + "(Empty) [" + (new Date()).getMilliseconds() + "]" + " : " + outResult);
		fpEvent = FP_EVENT_RETRY;
	}
	
	if (obj.Scan == "1")
	{
		document.getElementById("Scan").src = "Images\\Scan.JPG";
	}
	else
	{
		document.getElementById("Scan").src = TRANSPARENT;
	}
	
	if (obj.ScanStop == "1")
	{
		document.getElementById("Scan").src = TRANSPARENT;
	}

	if (obj.Monitor == "1")
	{
		document.getElementById("Monitor").src = "Images\\Monitor.JPG";
	}
	else
	{
		document.getElementById("Monitor").src = TRANSPARENT;
	}

}

function GetImageEx(dataInput)
{
	if(f_Thread == 0)
	{
		return;
	}
	GenericCommand(dataInput, GetImageFunc, GetImageErrFunc);	
}

function GetImageFunc(outResult)
{
	postTimeoutCount = 0;
	errorRetryCount = 0;

	try {
		obj = JSON.parse(outResult)
		updateImage(obj);
	}
	catch (e) {
		console.log(Date() + "[" + (new Date()).getMilliseconds() + "]" + " : " + outResult);
		fpEvent = FP_EVENT_RETRY;
	}
}

function GetImageErrFunc()
{
	postTimeoutCount = 0;
	fpEvent = FP_EVENT_IDLE;
}

function ErrorHandler(xhr)
{
	if(xhr.readyState == 0 && xhr.responseText == "")
	{
		errorRetryCount++;
		if(errorRetryCount >= ERROR_RETRY_COUNT)
		{
			errorRetryCount = ERROR_RETRY_COUNT;
			CloseRun(1);
		}
	}
	else
	{
		errorRetryCount = 0;
		console.log("Post Fail: " + xhr.status);
	}
}

function StartRun()
{
	if(runClosedFlag == 1)
	{
		runClosedFlag = 0;
		StartThread();
		
		runThreadId = setInterval(run, RUN_TASK_INTERVAL);
	}
}

function CloseRun(alertFlag)
{
	if(runClosedFlag == 0)
	{
		runClosedFlag = 1;
		
		if(alertFlag)
		{
			FailedCommunication(alertFlag);
		}
		clearInterval(runThreadId);
	}
}

function StartThread()
{
	var dataInput = {queryMode: "StartThread"};
	GenericCommand(dataInput, SetStartThread);
}

function SetStartThread()
{
	f_Thread = 1; 
}

function GenericCommand(dataIn, func, errorFunc)
{	
	var urlIn = "Include/IsStatusReady.asp";
	if(dataIn != "")
	{
		urlIn = "Include/WebControlMethod.asp";
	}

	$.ajax({
		url: urlIn,
		type: "POST",
		data: dataIn,
		success: function (outResult) {
				func(outResult);
			},
		error: function (xhr, status, error) { if(errorFunc != undefined){errorFunc();} ErrorHandler(xhr);},
		async: true
	});

}


